from django.db import models

from bleach.css_sanitizer import CSSSanitizer

from django_bleach.models import BleachField


class Person(models.Model):
    name = models.CharField(max_length=20)
    biography = BleachField(
        max_length=100,
        verbose_name='Person biography',
        allowed_tags=['p', 'a', 'li', 'ul', 'strong'],
        allowed_attributes=['class', 'href', 'style'],
        allowed_protocols=['http', 'https'],
        css_sanitizer=CSSSanitizer(allowed_css_properties=['color', 'background-color'])
    )



from django import forms
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.utils.safestring import mark_safe

import bleach
import warnings
from bleach.css_sanitizer import CSSSanitizer
from importlib import import_module

from django_bleach.utils import get_bleach_default_options


def load_widget(path):
    """ Load custom widget for the form field """
    i = path.rfind(".")
    module, attr = path[:i], path[i + 1:]
    try:
        mod = import_module(module)
    except (ImportError, ValueError) as e:
        error_message = "Error importing widget for BleachField %s: '%s'"
        raise ImproperlyConfigured(error_message % (path, e))

    try:
        cls = getattr(mod, attr)
    except AttributeError:
        raise ImproperlyConfigured(
            f"Module '{module}' does not define a '{attr}' widget"
        )

    return cls


def get_default_widget():
    """ Get the default widget or the widget defined in settings """
    default_widget = forms.Textarea
    if hasattr(settings, "BLEACH_DEFAULT_WIDGET"):
        default_widget = load_widget(settings.BLEACH_DEFAULT_WIDGET)
    return default_widget


class BleachField(forms.CharField):
    """ Bleach form field """
    empty_values = [None, "", [], (), {}]

    def __init__(self, allowed_tags=None, allowed_attributes=None,
                 allowed_styles=None, allowed_protocols=None,
                 strip_comments=None, strip_tags=None, css_sanitizer=None, *args, **kwargs):

        self.widget = get_default_widget()

        super().__init__(*args, **kwargs)

        self.bleach_options = get_bleach_default_options()

        if allowed_tags is not None:
            self.bleach_options["tags"] = allowed_tags
        if allowed_attributes is not None:
            self.bleach_options["attributes"] = allowed_attributes
        if allowed_styles is not None:
            warnings.warn(
                "allowed_styles will be deprecated, use css_sanitizer instead",
                DeprecationWarning
            )

            if css_sanitizer:
                warnings.warn("allowed_styles argument is ignored since css_sanitizer is favoured over allowed_styles")
            self.bleach_options["css_sanitizer"] = CSSSanitizer(allowed_css_properties=allowed_styles)
        if css_sanitizer is not None:
            self.bleach_options["css_sanitizer"] = css_sanitizer
        if allowed_protocols is not None:
            self.bleach_options["protocols"] = allowed_protocols
        if strip_tags is not None:
            self.bleach_options["strip"] = strip_tags
        if strip_comments is not None:
            self.bleach_options["strip_comments"] = strip_comments

    def to_python(self, value):
        """
        Strips any dodgy HTML tags from the input.

        Mark the return value as template safe.
        """
        if value in self.empty_values:
            return self.empty_value
        return mark_safe(bleach.clean(value, **self.bleach_options))



from django.db import OperationalError
from django.http import HttpResponseRedirect
from django.shortcuts import render

from .forms import BleachForm, PersonForm
from .models import Person


def home(request):
    if request.POST:
        form = BleachForm(request.POST)
        if form.is_valid():
            return HttpResponseRedirect(request.path + '?ok')
    else:
        form = BleachForm()

    return render(request, 'home.html', {'form': form})


def model_form(request):

    if request.POST:
        form = PersonForm(request.POST)
        if form.is_valid():
            form.save()
            return HttpResponseRedirect('?ok')
    else:
        form = PersonForm()
        try:
            people = list(Person.objects.all())
        except OperationalError:
            people = []

        return render(request, 'model_form.html', {
            'form': form,
            'people': people,
        })



from django import forms
from django.test import TestCase, override_settings
from django.utils.safestring import SafeString

from bleach.css_sanitizer import CSSSanitizer

from django_bleach.forms import BleachField
from testproject.constants import (
    ALLOWED_ATTRIBUTES,
    ALLOWED_CSS_PROPERTIES,
    ALLOWED_PROTOCOLS,
    ALLOWED_STYLES,
    ALLOWED_TAGS,
)
from testproject.forms import BleachForm, CustomBleachWidget


class TestBleachField(TestCase):

    def test_empty(self):
        """ Test that the empty_value arg is returned for any input empty value """
        for requested_empty_value in ('', None):
            field = BleachField(empty_value=requested_empty_value)
            for empty_value in field.empty_values:
                self.assertEqual(field.to_python(empty_value), requested_empty_value)

    def test_return_type(self):
        """ Test bleached values are SafeString objects """
        field = BleachField()
        self.assertIsInstance(field.to_python("some text"), SafeString)

    def test_deprecation_allowed_styles(self):
        with self.assertWarns(DeprecationWarning):
            BleachField(allowed_styles=ALLOWED_STYLES)

    def test_prefer_css_sanitizer_over_allowed_styles(self):
        with self.assertWarnsMessage(
            UserWarning,
            "allowed_styles argument is ignored since css_sanitizer is favoured over allowed_styles"
        ):
            field = BleachField(
                allowed_styles=['color', 'text-align'],
                css_sanitizer=CSSSanitizer(allowed_css_properties=ALLOWED_CSS_PROPERTIES)
            )
            self.assertIsInstance(field.bleach_options["css_sanitizer"], CSSSanitizer)
            self.assertEqual(field.bleach_options["css_sanitizer"].allowed_css_properties, ALLOWED_CSS_PROPERTIES)

    def test_bleaching(self):
        """ Test values are bleached """
        test_data = {
            'no_tags': "<h1>Heading</h1>",
            'no_strip': "<h1>Heading</h1>",
            'bleach_strip': "<!-- script here -->"
                            "<script>alert(\"Hello World\")</script>",
            'bleach_attrs': "<a href=\"https://www.google.com\" "
                            "target=\"_blank\">google.com</a>",
            'bleach_css_sanitizer': "<li style=\"color: white\">item</li>",
        }
        form = BleachForm(data=test_data)
        form.is_valid()
        self.assertEqual(
            form.cleaned_data['no_tags'], 'Heading'
        )
        self.assertEqual(
            form.cleaned_data['no_strip'],
            '&lt;h1&gt;Heading&lt;/h1&gt;'
        )
        self.assertEqual(
            form.cleaned_data['bleach_strip'],
            'alert("Hello World")'
        )
        self.assertEqual(
            form.cleaned_data['bleach_attrs'],
            '<a href="https://www.google.com">google.com</a>'
        )
        self.assertNotEqual(
            form.cleaned_data['bleach_css_sanitizer'],
            test_data['bleach_css_sanitizer']
        )

    def test_tags(self):
        """ Test allowed tags are rendered"""
        test_data = {
            'no_tags': "<p>No tags here</p>",
            'no_strip': "No tags here",
            'bleach_strip': "<ul><li>one</li><li>two</li></ul>",
            'bleach_attrs': "<a href=\"https://www.google.com\" "
                            "title=\"Google\">google.com</a>",
            'bleach_css_sanitizer': "<li style=\"color: white;\">item</li>"
        }
        form = BleachForm(data=test_data)
        form.is_valid()
        self.assertEqual(form.cleaned_data['no_tags'], "No tags here")
        self.assertEqual(form.cleaned_data['no_strip'], "No tags here")

        self.assertEqual(
            form.cleaned_data['bleach_strip'],
            test_data['bleach_strip']
        )
        self.assertEqual(
            form.cleaned_data['bleach_attrs'],
            test_data['bleach_attrs']
        )
        self.assertEqual(
            form.cleaned_data['bleach_css_sanitizer'],
            test_data['bleach_css_sanitizer']
        )

    def test_attrs(self):
        """ Test allowed attributes are rendered """
        list_html = "<ul class=\"our-list\">" \
                    "<li class=\"list-item\">one</li>" \
                    "<li>two</li>" \
                    "</ul>"
        test_data = {
            'no_strip': "",
            'no_tags': list_html,
            'bleach_strip': list_html,
            'bleach_attrs': list_html,
            'bleach_css_sanitizer': list_html
        }
        form = BleachForm(data=test_data)
        form.is_valid()
        self.assertEqual(form.cleaned_data['no_tags'], '\none\ntwo')

        self.assertEqual(
            form.cleaned_data['bleach_strip'],
            '<ul><li>one</li><li>two</li></ul>'
        )
        self.assertEqual(
            form.cleaned_data['bleach_attrs'],
            test_data['bleach_strip']
        )
        self.assertEqual(
            form.cleaned_data['bleach_css_sanitizer'],
            '<ul><li>one</li><li>two</li></ul>'
        )


@override_settings(BLEACH_DEFAULT_WIDGET='testproject.forms.CustomBleachWidget')
class TestCustomWidget(TestCase):

    def setUp(self):
        class CustomForm(forms.Form):
            # Define form inside function with overridden settings so
            # get_default_widget() sees the modified setting.
            no_tags = BleachField(
                max_length=100,
                strip_tags=True,
                allowed_tags=[]
            )
            no_strip = BleachField(
                max_length=100,
                allowed_tags=None,
                allowed_attributes=None
            )
            bleach_strip = BleachField(
                max_length=100,
                strip_comments=True,
                strip_tags=True,
                allowed_tags=ALLOWED_TAGS
            )
            bleach_attrs = BleachField(
                max_length=100,
                strip_tags=False,
                allowed_tags=ALLOWED_TAGS,
                allowed_protocols=ALLOWED_PROTOCOLS,
                allowed_attributes=ALLOWED_ATTRIBUTES
            )
            bleach_styles = BleachField(
                max_length=100,
                strip_tags=False,
                allowed_attributes=['style'],
                allowed_tags=ALLOWED_TAGS,
                allowed_styles=ALLOWED_STYLES
            )
            bleach_css_sanitizer = BleachField(
                max_length=100,
                strip_tags=False,
                allowed_attributes=['style'],
                allowed_tags=ALLOWED_TAGS,
                css_sanitizer=CSSSanitizer(allowed_css_properties=ALLOWED_CSS_PROPERTIES)
            )
        self.CustomForm = CustomForm

    def test_custom_widget_type(self):
        """ Test widget class matches BLEACH_DEFAULT_WIDGET """
        for field in self.CustomForm().fields.values():
            self.assertIsInstance(field.widget, CustomBleachWidget)

    def test_custom_widget_bleaches_content(self):
        """
        Test input is bleached according to config while using a custom
        widget
        """
        test_data = {
            'no_tags': '<h1>Heading</h1>',
            'no_strip': '<h1>Heading</h1>',
            'bleach_strip': '<!-- script here -->'
                            '<script>alert("Hello World")</script>',
            'bleach_attrs': (
                '<a href="http://www.google.com" '
                'target="_blank">google.com</a>'
                '<a href="https://www.google.com">google.com</a>'
            ),
            'bleach_styles': '<li style="color: white">item</li>',
            'bleach_css_sanitizer': '<li style="color: white">item</li>'
        }
        form = self.CustomForm(data=test_data)
        form.is_valid()
        self.assertEqual(
            form.cleaned_data['no_tags'], 'Heading'
        )
        self.assertEqual(
            form.cleaned_data['no_strip'],
            '&lt;h1&gt;Heading&lt;/h1&gt;'
        )
        self.assertEqual(
            form.cleaned_data['bleach_strip'],
            'alert("Hello World")'
        )
        self.assertEqual(
            form.cleaned_data['bleach_attrs'],
            '<a>google.com</a><a href="https://www.google.com">google.com</a>'
        )
        self.assertNotEqual(
            form.cleaned_data['bleach_styles'],
            test_data['bleach_styles']
        )
        self.assertNotEqual(
            form.cleaned_data['bleach_css_sanitizer'],
            test_data['bleach_css_sanitizer']
        )



import os
import sys


SITE_ID = 1

PROJECT_PATH = os.path.abspath(os.path.dirname(__file__))

PYTHON_VERSION = '%s.%s' % sys.version_info[:2]


DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(PROJECT_PATH, 'django-bleach.db')
    }
}
DEFAULT_AUTO_FIELD = 'django.db.models.AutoField'
DATABASE_SUPPORTS_TRANSACTIONS = True

INSTALLED_APPS = [
    'django.contrib.auth',
    'django.contrib.admin',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.messages',

    'django_bleach',
    'testproject'
]

LANGUAGE_CODE = "en"

LANGUAGES = (
    ('en', 'English'),
)

ROOT_URLCONF = 'testproject.urls'

DEBUG = True

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'APP_DIRS': True,
        'OPTIONS': {
            'debug': False,
            'context_processors': (
                "django.contrib.auth.context_processors.auth",
                "django.template.context_processors.debug",
                "django.template.context_processors.i18n",
                "django.template.context_processors.media",
                "django.template.context_processors.request",
                "django.template.context_processors.static",
                "django.template.context_processors.tz",
                "django.contrib.messages.context_processors.messages"
            )
        },
        'DIRS': ('templates', )
    },
]

USE_TZ = True
SECRET_KEY = 'blah'

MIDDLEWARE = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware'
)

# BLEACH_DEFAULT_WIDGET = 'testproject.forms.CustomBleachWidget'



from django import forms

from bleach.css_sanitizer import CSSSanitizer

from django_bleach.forms import BleachField
from testproject.constants import (
    ALLOWED_ATTRIBUTES,
    ALLOWED_CSS_PROPERTIES,
    ALLOWED_PROTOCOLS,
    ALLOWED_STYLES,
    ALLOWED_TAGS,
)
from testproject.models import Person


class CustomBleachWidget(forms.Textarea):

    def __init__(self, attrs=None):
        default_attrs = {'rows': 15, 'cols': 60}
        default_attrs.update(attrs or {})
        super().__init__(attrs=default_attrs)


class BleachForm(forms.Form):
    """ Form for testing BleachField """
    no_tags = BleachField(
        max_length=100,
        strip_tags=True,
        allowed_tags=[]
    )

    no_strip = BleachField(
        max_length=100,
        allowed_tags=None,
        allowed_attributes=None
    )

    bleach_strip = BleachField(
        max_length=100,
        strip_comments=True,
        strip_tags=True,
        allowed_tags=ALLOWED_TAGS
    )
    bleach_attrs = BleachField(
        max_length=100,
        strip_tags=False,
        allowed_tags=ALLOWED_TAGS,
        allowed_protocols=ALLOWED_PROTOCOLS,
        allowed_attributes=ALLOWED_ATTRIBUTES
    )
    bleach_styles = BleachField(
        max_length=100,
        strip_tags=False,
        allowed_attributes=['style'],
        allowed_tags=ALLOWED_TAGS,
        allowed_styles=ALLOWED_STYLES
    )
    bleach_css_sanitizer = BleachField(
        max_length=100,
        strip_tags=False,
        allowed_attributes=['style'],
        allowed_tags=ALLOWED_TAGS,
        css_sanitizer=CSSSanitizer(allowed_css_properties=ALLOWED_CSS_PROPERTIES)
    )


class PersonForm(forms.ModelForm):

    class Meta:
        model = Person
        fields = '__all__'



from django import forms
from django.test import TestCase, override_settings

from django_bleach import forms as bleach_forms
from testproject.forms import CustomBleachWidget
from testproject.models import Person

from .test_models import BleachContent


class BleachContentModelForm(forms.ModelForm):
    class Meta:
        model = BleachContent
        fields = '__all__'


class TestModelFormField(TestCase):

    def setUp(self):
        model_form = BleachContentModelForm()
        self.form_field = model_form.fields['content']
        self.choice_form_field = model_form.fields['choice']
        self.blank_field_form_field = model_form.fields['blank_field']
        self.model_field = BleachContent()._meta.get_field('content')

    def test_formfield_type(self):
        """
        Check content's form field is instance of BleachField
        """
        self.assertIsInstance(self.form_field, bleach_forms.BleachField)

    def test_default_widget_type(self):
        """
        Widget class is Textarea when BLEACH_DEFAULT_WIDGET is not set.
        """
        form = forms.modelform_factory(Person, fields='__all__')()
        self.assertIsInstance(
            form.fields['biography'].widget, forms.Textarea,
        )

    @override_settings(
        BLEACH_DEFAULT_WIDGET='testproject.forms.CustomBleachWidget'
    )
    def test_custom_widget_type(self):
        """
        Widget class matches BLEACH_DEFAULT_WIDGET setting.
        """
        form = forms.modelform_factory(Person, fields='__all__')()
        self.assertIsInstance(
            form.fields['biography'].widget, CustomBleachWidget
        )

    @override_settings(
        BLEACH_DEFAULT_WIDGET='testproject.forms.CustomBleachWidget'
    )
    def test_widget_override(self):
        """
        Widget class matches widget class specified in overrides.
        """
        form = forms.modelform_factory(
            Person,
            fields='__all__',
            widgets={"biography": CustomBleachWidget},
        )()
        self.assertIsInstance(
            form.fields['biography'].widget, CustomBleachWidget
        )

    def test_same_allowed_args(self):
        """
        Check model and form's allowed arguments (tags, attributes, ...) are same
        """
        form_allowed_args: dict = self.form_field.bleach_options
        model_allowed_args: dict = self.model_field.bleach_kwargs

        self.assertEqual(model_allowed_args, form_allowed_args)

    def test_with_choices(self):
        """
        Check if choices specified, use TextField's default widget (Select).
        """
        form_field_widget = self.choice_form_field.widget.__class__
        self.assertEqual(form_field_widget, forms.widgets.Select)

    def test_optional_field(self):
        """
        Check for the required flag on fields with `blank=True`
        """
        self.assertEqual(self.blank_field_form_field.required, False)

    def test_required_field(self):
        """
        Check for the required flag on fields
        """
        self.assertEqual(self.form_field.required, True)


class CustomBleachedFormField(bleach_forms.BleachField):
    ...


class OverriddenBleachContentModelForm(forms.ModelForm):
    class Meta:
        model = BleachContent
        fields = '__all__'
        field_classes = {
            "content": CustomBleachedFormField,
        }


class TestModelFormFieldOverrides(TestCase):

    def setUp(self):
        model_form = OverriddenBleachContentModelForm()
        self.form_field = model_form.fields['content']

    def test_formfield_type(self):
        """
        Check content's form field is instance of CustomBleachedFormField.
        """
        self.assertIsInstance(self.form_field, CustomBleachedFormField)



from django.db import models
from django.utils.safestring import mark_safe

import warnings
from bleach import clean
from bleach.css_sanitizer import CSSSanitizer

from . import forms
from .utils import get_bleach_default_options


class BleachField(models.TextField):
    def __init__(self, allowed_tags=None, allowed_attributes=None,
                 allowed_styles=None, allowed_protocols=None,
                 strip_tags=None, strip_comments=None, css_sanitizer=None, *args, **kwargs):

        super().__init__(*args, **kwargs)

        self.bleach_kwargs = get_bleach_default_options()

        if allowed_tags:
            self.bleach_kwargs["tags"] = allowed_tags
        if allowed_attributes:
            self.bleach_kwargs["attributes"] = allowed_attributes
        if allowed_styles:
            warnings.warn(
                "allowed_styles will be deprecated, use css_sanitizer instead",
                DeprecationWarning
            )
            if css_sanitizer:
                warnings.warn("allowed_styles argument is ignored since css_sanitizer is favoured over allowed_styles")
            self.bleach_kwargs["css_sanitizer"] = CSSSanitizer(allowed_css_properties=allowed_styles)
        if css_sanitizer:
            self.bleach_kwargs["css_sanitizer"] = css_sanitizer
        if allowed_protocols:
            self.bleach_kwargs["protocols"] = allowed_protocols
        if strip_tags:
            self.bleach_kwargs["strip"] = strip_tags
        if strip_comments:
            self.bleach_kwargs["strip_comments"] = strip_comments

    def formfield(self, form_class=forms.BleachField, **kwargs):
        """ Makes the field for a ModelForm """

        # If field doesn't have any choices add kwargs expected by BleachField.
        if not self.choices:
            kwargs.setdefault("widget", forms.get_default_widget())
            kwargs.update({
                "max_length": self.max_length,
                "allowed_tags": self.bleach_kwargs.get("tags"),
                "allowed_attributes": self.bleach_kwargs.get("attributes"),
                "css_sanitizer": self.bleach_kwargs.get("css_sanitizer"),
                "allowed_protocols": self.bleach_kwargs.get("protocols"),
                "strip_tags": self.bleach_kwargs.get("strip"),
                "strip_comments": self.bleach_kwargs.get("strip_comments"),
                "required": not self.blank,
            })

        return super().formfield(form_class=form_class, **kwargs)

    def pre_save(self, model_instance, add):
        data = getattr(model_instance, self.attname)
        if data is None:
            return data
        clean_value = clean(data, **self.bleach_kwargs) if data else ""
        setattr(model_instance, self.attname, mark_safe(clean_value))
        return clean_value

    def from_db_value(self, value, expression, connection):
        if value is None:
            return value
        # Values are sanitised before saving, so any value returned from the DB
        # is safe to render unescaped.
        return mark_safe(value)



ALLOWED_ATTRIBUTES = {
    '*': ['class', 'style'],
    'a': ['href', 'title']
}

ALLOWED_CSS_PROPERTIES = [
    'color'
]

ALLOWED_PROTOCOLS = [
    'https',
    'data',
]

ALLOWED_STYLES = ALLOWED_CSS_PROPERTIES

ALLOWED_TAGS = [
    'a',
    'li',
    'ul'
]



from django import template
from django.utils.safestring import mark_safe

import bleach

from django_bleach.utils import get_bleach_default_options


register = template.Library()


@register.filter(name="bleach")
def bleach_value(value, tags=None):
    if value is None:
        return None

    bleach_args = get_bleach_default_options()
    if tags is not None:
        args = bleach_args.copy()
        args["tags"] = tags.split(",")
    else:
        args = bleach_args
    bleached_value = bleach.clean(value, **args)
    return mark_safe(bleached_value)


@register.filter
def bleach_linkify(value):
    """
    Convert URL-like strings in an HTML fragment to links

    This function converts strings that look like URLs, domain names and email
    addresses in text that may be an HTML fragment to links, while preserving:

        1. links already in the string
        2. urls found in attributes
        3. email addresses
    """
    if value is None:
        return None

    return bleach.linkify(value, parse_email=True)



#!/usr/bin/env python
import codecs
import os
import re
import sys
from setuptools import find_packages, setup
from setuptools.command.test import test as test_command


try:
    from sphinx.setup_command import BuildDoc
except ImportError:
    BuildDoc = None


class Tox(test_command):
    user_options = [('tox-args=', 'a', "Arguments to pass to tox")]

    def initialize_options(self):
        test_command.initialize_options(self)
        self.tox_args = None

    def finalize_options(self):
        test_command.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        # import here, cause outside the eggs aren't loaded
        import shlex
        import tox
        args = self.tox_args
        if args:
            args = shlex.split(self.tox_args)
        errno = tox.cmdline(args=args)
        sys.exit(errno)


def read(*parts):
    file_path = os.path.join(os.path.dirname(__file__), *parts)
    return codecs.open(file_path, encoding='utf-8').read()


def find_variable(variable, *parts):
    version_file = read(*parts)
    version_match = re.search(
        fr"^{variable} = ['\"]([^'\"]*)['\"]", version_file, re.M
    )
    if version_match:
        return str(version_match.group(1))
    raise RuntimeError("Unable to find version string.")


name = 'django-bleach'
release = find_variable('__version__', 'django_bleach', '__init__.py')
version = release.rstrip('.')

setup(
    name=name,
    version=version,
    description='Easily use bleach with Django models and templates',
    long_description=read('README.rst'),
    long_description_content_type='text/x-rst',
    author='Tim Heap',
    maintainer='Mark Walker',
    maintainer_email='theshow+django-bleach@gmail.com',
    url='https://github.com/marksweb/django-bleach',
    license='MIT',
    packages=find_packages(exclude=('testproject*',)),
    install_requires=[
        'bleach[css]>=5,<6',
        'Django>=3.2',
    ],
    python_requires='>=3.8',
    tests_require=[
        'bleach[css]>=5,<6',
        'mock',
        'sphinx',
        'tox'
    ],
    cmdclass={
        'build_sphinx': BuildDoc,
        'test': Tox
    },
    command_options={
        'build_sphinx': {
            'project': ('setup.py', name),
            'version': ('setup.py', version),
            'release': ('setup.py', release),
            'source_dir': ('setup.py', 'docs'),
            'build_dir': ('setup.py', './docs/_build')
        }
    },
    package_data={},
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Framework :: Django :: 3.2',
        'Framework :: Django :: 4.0',
        'Framework :: Django :: 4.1',
        'Development Status :: 5 - Production/Stable',
    ],
    project_urls={
        'Documentation': 'https://django-bleach.readthedocs.io/',
        'Release notes': 'https://github.com/marksweb/django-bleach/blob/main/CHANGELOG.md',
        'Issues': 'https://github.com/marksweb/django-bleach/issues',
        'Source': 'https://github.com/marksweb/django-bleach',
    }
)



# Generated by Django 3.2.4 on 2021-06-15 12:39

from django.db import migrations, models
import django_bleach.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Person',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=20)),
                ('biography', django_bleach.models.BleachField(max_length=100, verbose_name='Person biography')),
            ],
        ),
    ]



from django.db import models
from django.test import TestCase
from django.utils.safestring import SafeString

from bleach.css_sanitizer import CSSSanitizer

from django_bleach.models import BleachField
from testproject.constants import (
    ALLOWED_ATTRIBUTES,
    ALLOWED_CSS_PROPERTIES,
    ALLOWED_PROTOCOLS,
    ALLOWED_STYLES,
    ALLOWED_TAGS,
)


class BleachContent(models.Model):
    """ Bleach test model"""
    CHOICES = (
        ('f', 'first choice'),
        ('s', 'second choice')
    )
    content = BleachField(
        allowed_attributes=ALLOWED_ATTRIBUTES,
        allowed_protocols=ALLOWED_PROTOCOLS,
        css_sanitizer=CSSSanitizer(allowed_css_properties=ALLOWED_CSS_PROPERTIES),
        allowed_tags=ALLOWED_TAGS,
        strip_comments=True,
        strip_tags=True
    )
    choice = BleachField(choices=CHOICES)
    blank_field = BleachField(blank=True)
    null_field = BleachField(blank=True, null=True)


class TestBleachModelField(TestCase):
    """ Test model field """

    def test_bleaching(self):
        """ Test values are bleached """
        test_data = {
            'no_tags': "<h1>Heading</h1>",
            'no_strip': "<h1>Heading</h1>",
            'bleach_strip': """<script>alert("Hello World")</script>""",
            'bleach_attrs': "<a href=\"http://www.google.com\" "
                            "target=\"_blank\">google.com</a>",
            'bleach_css_sanitizer': "<li style=\"color: white\">item</li>",
            'bleach_comment': "<!-- this is a comment -->",
        }
        expected_values = {
            'no_tags': "Heading",
            'no_strip': "Heading",
            'bleach_strip': """alert("Hello World")""",
            'bleach_attrs':
                "<a>google.com</a>",
            'bleach_css_sanitizer': "<li style=\"color: white;\">item</li>",
            'bleach_comment': ""
        }

        for key, value in test_data.items():
            obj = BleachContent.objects.create(content=value)
            self.assertEqual(obj.content, expected_values[key])

    def test_deprecation_allowed_styles(self):
        with self.assertWarns(DeprecationWarning):
            BleachField(allowed_styles=ALLOWED_STYLES)

    def test_prefer_css_sanitizer_over_allowed_styles(self):
        with self.assertWarnsMessage(
            UserWarning,
            "allowed_styles argument is ignored since css_sanitizer is favoured over allowed_styles"
        ):
            field = BleachField(allowed_styles=['color', 'text-align'],
                                css_sanitizer=CSSSanitizer(allowed_css_properties=ALLOWED_CSS_PROPERTIES))
            self.assertIsInstance(field.bleach_kwargs["css_sanitizer"], CSSSanitizer)
            self.assertEqual(field.bleach_kwargs["css_sanitizer"].allowed_css_properties, ALLOWED_CSS_PROPERTIES)

    def test_retrieved_values_are_template_safe(self):
        obj = BleachContent.objects.create(content="some content")
        obj.refresh_from_db()
        self.assertIsInstance(obj.content, SafeString)
        obj = BleachContent.objects.create(content="")
        obj.refresh_from_db()
        self.assertIsInstance(obj.content, SafeString)

    def test_saved_values_are_template_safe(self):
        obj = BleachContent(content="some content")
        obj.save()
        self.assertIsInstance(obj.content, SafeString)
        obj = BleachContent(content="")
        obj.save()
        self.assertIsInstance(obj.content, SafeString)

    def test_saved_none_values_are_none(self):
        obj = BleachContent(null_field=None)
        obj.save()
        self.assertIsNone(obj.null_field)


class BleachNullableContent(models.Model):
    """ Bleach test model"""
    content = BleachField(
        allowed_attributes=ALLOWED_ATTRIBUTES,
        allowed_protocols=ALLOWED_PROTOCOLS,
        css_sanitizer=CSSSanitizer(allowed_css_properties=ALLOWED_CSS_PROPERTIES),
        allowed_tags=ALLOWED_TAGS,
        strip_comments=True,
        strip_tags=True,
        blank=True,
        null=True
    )


class TestBleachNullableModelField(TestCase):
    """ Test model field """

    def test_bleaching(self):
        """ Test values are bleached """
        test_data = {
            'none': None,
            'empty': "",
            'whitespaces': "   ",
            'linebreak': "\n",
        }
        expected_values = {
            'none': None,
            'empty': "",
            'whitespaces': "   ",
            'linebreak': "\n",
        }

        for key, value in test_data.items():
            obj = BleachNullableContent.objects.create(content=value)
            self.assertEqual(obj.content, expected_values[key])



from django.template import Context, Template
from django.test import TestCase


class TestBleachTemplates(TestCase):
    """ Test template tags """

    def test_bleaching(self):
        """ Test that unsafe tags are sanitised """
        context = Context(
            {'some_unsafe_content': '<script>alert("Hello World!")</script>'},
        )
        template_to_render = Template(
            '{% load bleach_tags %}'
            '{{ some_unsafe_content|bleach }}'
        )
        rendered_template = template_to_render.render(context)
        self.assertInHTML(
            '&lt;script&gt;alert("Hello World!")&lt;/script&gt;',
            rendered_template
        )

    def test_bleaching_none(self):
        """ Test that None is handled properly as an input """
        context = Context(
            {'none_value': None}
        )
        template_to_render = Template(
            '{% load bleach_tags %}'
            '{{ none_value|bleach }}'
        )
        rendered_template = template_to_render.render(context)
        self.assertEqual(
            'None',
            rendered_template
        )

    def test_bleaching_tags(self):
        """ Test provided tags are kept """
        context = Context(
            {'some_unsafe_content': '<script>alert("Hello World!")</script>'}
        )
        template_to_render = Template(
            '{% load bleach_tags %}'
            '{{ some_unsafe_content|bleach:"script" }}'
        )
        rendered_template = template_to_render.render(context)
        self.assertInHTML(
            '<script>alert("Hello World!")</script>', rendered_template
        )

    def test_linkify(self):
        """ Test bleach linkify """
        url = 'www.google.com'
        context = Context({'link_this': url})
        template_to_render = Template(
            '{% load bleach_tags %}'
            '{{ link_this|bleach_linkify|safe }}'
        )
        rendered_template = template_to_render.render(context)
        self.assertInHTML(
            '<a href="http://{0}" rel="nofollow">{0}</a>'.format(url),
            rendered_template
        )

    def test_linkify_none(self):
        """ Test bleach linkify with None as an input """
        context = Context({'none_value': None})
        template_to_render = Template(
            '{% load bleach_tags %}'
            '{{ none_value|bleach_linkify }}'
        )
        rendered_template = template_to_render.render(context)
        self.assertEqual(
            'None',
            rendered_template,
        )
